<?php

namespace Acms\Services\SocialLogin;

use Facebook\Facebook as FacebookSDK;

class Facebook extends Base
{
    /**
     * @var \Facebook\Facebook
     */
    protected $sdk;

    /**
     * @var string
     */
    protected $appId;

    /**
     * @var \Facebook\GraphNodes\GraphUser
     */
    protected $me;

    /**
     * @var string
     */
    protected $accessToken;

    /**
     * Facebook constructor.
     * @param string $appId
     * @param string $appSecret
     * @param string $version
     * @throws \Facebook\Exceptions\FacebookSDKException
     */
    public function __construct($appId, $appSecret, $version)
    {
        $this->appId = $appId;
        $this->sdk = new FacebookSDK(array(
            'app_id' => $appId,
            'app_secret' => $appSecret,
            'default_graph_version' => $version,
        ));
    }

    /**
     * Get login url.
     *
     * @param string $url
     * @param array $permissions
     * @return string
     */
    public function getLoginUrl($url, $permissions = array('email'))
    {
        $helper = $this->sdk->getRedirectLoginHelper();
        return $helper->getLoginUrl($url, $permissions);
    }

    /**
     * @return string
     * @throws \Facebook\Exceptions\FacebookSDKException
     */
    public function getAccessToken()
    {
        if ($this->accessToken) {
            return $this->accessToken;
        }
        $helper = $this->sdk->getRedirectLoginHelper();
        $accessToken = $helper->getAccessToken();
        if (!isset($accessToken)) {
            if ($helper->getError()) {
                throw new \RuntimeException($helper->getError());
            }
            throw new \RuntimeException('Bad request.');
        }
        $oAuth2Client = $this->sdk->getOAuth2Client();

        $tokenMetadata = $oAuth2Client->debugToken($accessToken);
        $tokenMetadata->validateAppId($this->appId);
        $tokenMetadata->validateExpiration();

        if (! $accessToken->isLongLived()) {
            $accessToken = $oAuth2Client->getLongLivedAccessToken($accessToken);
        }
        $this->accessToken = $accessToken->getValue();
        return $this->accessToken;
    }

    public function setMe()
    {
        $accessToken = $this->getAccessToken();
        $response = $this->sdk->get('/me?fields=id,name,email', $accessToken);
        $this->me = $response->getGraphUser();
    }

    /**
     * @return mixed
     */
    protected function getId()
    {
        return $this->me->getId();
    }

    /**
     * @return mixed
     */
    protected function getEmail()
    {
        $email = $this->me->getEmail();
        if (empty($email)) {
            $email = $this->getId() . '@example.com';
        }
        return $email;
    }

    /**
     * @return mixed
     */
    protected function getName()
    {
        return $this->me->getName();
    }

    /**
     * @return mixed
     */
    protected function getCode()
    {
        return $this->me->getId();
    }

    /**
     * @return mixed
     */
    protected function getIcon()
    {
        $image_uri = 'https://graph.facebook.com/' . $this->getId() . '/picture?type=large';
        return $this->userIconFromUri($image_uri);
    }

    /**
     * @return mixed
     */
    protected function getUserKey()
    {
        return 'user_facebook_id';
    }
}
